﻿Imports _4_PhoneBook.Model

Public Class ContactsDataProvider

    Private ReadOnly xmlDirectoryPath As String
    Friend Const DataFolderName As String = "Data"
    Private ReadOnly xmlFileName As String = "PhoneContacts"

    Public Sub New()
        xmlDirectoryPath = $"{AppDomain.CurrentDomain.BaseDirectory}\{DataFolderName}"
    End Sub

    Public Sub New(path As String, Optional overrideXmlFilename As String = Nothing)
        If Not IO.Directory.Exists(path) Then
            Throw New IO.DirectoryNotFoundException("Der Pfad zur XML existiert nicht oder wurde nicht korrekt angegeben.")
        End If
        If overrideXmlFilename IsNot Nothing Then
            xmlFileName = overrideXmlFilename
        End If
        xmlDirectoryPath = path
    End Sub

    Public ReadOnly Property XmlPath As String
        Get
            Return $"{xmlDirectoryPath}\{xmlFileName}.xml"
        End Get
    End Property

    ''' <summary>
    ''' Ruf alle gespeicherten Kontakte ab und sortiert diese wenn gewünscht in einem Rutsch
    ''' </summary>
    ''' <param name="sorted">Gibt an ob nach Nachnamen sortiert werden soll</param>
    ''' <returns>Gibt eine Auflistung an Kontakten zurück. Sind keine Kontakte vorhanden wird eine Dummy-Liste erstellt</returns>
    Public Function GetAllContacts(sorted As Boolean) As ICollection(Of Model.Contact)
        If Not IO.File.Exists(XmlPath) Then 'Wenn File nicht existiert dann ein neues File mit Beispieldaten erstellen
            Dim testContacts As List(Of Model.Contact) = SeedData()
            Dim ser = New Serializer.XmlSerializer()
            ser.Serialize(XmlPath, testContacts)

            If sorted Then
                Return testContacts.OrderBy(Function(o) o.LastName).ToList()
            Else
                Return testContacts
            End If
        Else
            Dim ser = New Serializer.XmlSerializer().DeSerialize(XmlPath, New List(Of Contact))
            If sorted Then
                Return ser.OrderBy(Function(o) o.LastName).ToList()
            Else
                Return ser
            End If
        End If
    End Function

    Public Sub SaveContacts(contacts As List(Of Contact))
        If contacts Is Nothing Then Throw New NullReferenceException()
        If Not IO.File.Exists(XmlPath) Then
            Throw New IO.FileNotFoundException($"Die XML kann nicht gefunden werden, bitte prüfe den Pfad. Angegebener Pfad: '{XmlPath}'")
        End If

        Dim ser As New Serializer.XmlSerializer()
        ser.Serialize(Of List(Of Contact))(XmlPath, contacts)
    End Sub

    Private Function SeedData() As List(Of Contact)
        Dim retList As New List(Of Contact)

        Dim cont1 As New Contact("Sascha", "Patschka", "https://www.vb-paradise.de/wcf/images/avatars/01/3128-0124b76125128a33003a968a4d2882f1479dd93f.jpg",
        New Date(1983, 9, 12), "Das bin ich ;-)")
        cont1.ContactData.Add(New ContactData("+43 664 1234567", EnumContactDataType.Phonenumber))
        cont1.ContactData.Add(New ContactData("patschka.sascha@live.com", EnumContactDataType.Mail))
        cont1.ContactData.Add(New ContactData("https://www.facebook.com/sascha.patschkavb", EnumContactDataType.SocialMedia))
        cont1.ContactData.Add(New ContactData("https://www.vb-paradise.de/index.php/User/26037-Nofear23m/?s=0e544e5122d3fd592f8ccff79c59f46763a2af95", EnumContactDataType.Website))
        retList.Add(cont1)

        Dim cont2 As New Contact("Marny", "Marks", "https://png.pngtree.com/png-clipart/20190924/original/pngtree-user-vector-avatar-png-image_4830521.jpg",
                New Date(1999, 1, 1), Nothing)
        cont2.ContactData.Add(New ContactData("+49 123 1234567", EnumContactDataType.Phonenumber))
        retList.Add(cont2)

        Dim cont3 As New Contact("Julian", "Kent", "https://png.pngtree.com/png-clipart/20190924/original/pngtree-user-vector-avatar-png-image_4830521.jpg",
                New Date(1999, 1, 1), "")
        cont3.ContactData.Add(New ContactData("+49 123 1234567", EnumContactDataType.Phonenumber))
        retList.Add(cont3)

        Dim cont4 As New Contact("Maxwell", "Rowland", "https://png.pngtree.com/png-clipart/20190924/original/pngtree-user-vector-avatar-png-image_4830521.jpg",
                New Date(1999, 1, 1), "")
        cont4.ContactData.Add(New ContactData("+49 123 1234567", EnumContactDataType.Phonenumber))
        retList.Add(cont4)

        Dim cont5 As New Contact("Olympia", "Mcintosh", "https://png.pngtree.com/png-clipart/20190924/original/pngtree-user-vector-avatar-png-image_4830521.jpg",
               New Date(1999, 1, 1), "")
        cont5.ContactData.Add(New ContactData("+49 123 1234567", EnumContactDataType.Phonenumber))
        retList.Add(cont5)

        Return retList
    End Function




End Class
